from datetime import datetime

from pandas import DataFrame

from offline_results.common.config import VISIONPLUS_DEV, VIEWED_DATA_PATH, CONFIG_HOMEPAGE_PAYTV, \
    CONFIG_HOMEPAGE_NO_PAYTV
from offline_results.common.constants import PAY_TV_CONTENT, NO_PAY_TV_CONTENT, TV_CHANNEL_LIST, CONTENT_ID, \
    HOMEPAGE_ID, MODEL_POPULAR, POPULAR, SERVICE_NAME, ALL_CONTENT_BASED, POPULAR_MODULE_DEFAULT, RECORDS, \
    PAY_TV, NAN, LEFT, ACTIVE, CONTENT_STATUS, IS_PAY_TV, CREATED_ON, CLUSTER_ID, SCORE, VIEW_COUNT, SUM, \
    HOMEPAGE_ID_BASED
from offline_results.common.read_write_from_s3 import ConnectS3
from offline_results.repository.graph_db_connection import ANGraphDb
from offline_results.similarity.content_profile.query_utils import QueryUtils
from offline_results.utils.logger import Logging

graph = ANGraphDb.new_connection_config().graph


class PopularModelFallback:
    @staticmethod
    def get_homepage_for_contents(
            is_pay_tv
    ) -> list:

        query = f'''g.V().hasLabel('{PAY_TV_CONTENT if is_pay_tv else NO_PAY_TV_CONTENT}' ).match(
            __.as("c").values().as("content_id"),
            __.as("c").out("HAS_HOMEPAGE")
            .values("homepage_id").as("homepage_id")
            ).select("content_id", "homepage_id")'''

        data = graph.custom_query(query, payload={PAY_TV_CONTENT: PAY_TV_CONTENT,
                                                  NO_PAY_TV_CONTENT: NO_PAY_TV_CONTENT})
        data = [rec for idx in data for rec in idx]
        df = DataFrame(data)
        return df

    @staticmethod
    def get_viewed_from_s3():
        try:
            cont = ConnectS3()
            resource = cont.create_connection()
            viewed_df = cont.read_compress_pickles_from_S3(
                bucket_name=VISIONPLUS_DEV,
                object_name=VIEWED_DATA_PATH,
                resource=resource
            )
            return viewed_df

        except Exception as e:
            Logging.error(f"Error while reading log from S3, Error: {e}")

    @staticmethod
    def get_default_popular_all_content(pay_tv_label):
        df = PopularModelFallback.get_viewed_from_s3()
        df_pay_tv = df[(df[IS_PAY_TV] == True if pay_tv_label == PAY_TV else False) & (df[CONTENT_STATUS] == ACTIVE)]
        df_pay_tv = df_pay_tv.merge(
            PopularModelFallback.get_homepage_for_contents(True if pay_tv_label == PAY_TV else False),
            on=CONTENT_ID, how=LEFT)
        df_popular = DataFrame(
            CONFIG_HOMEPAGE_PAYTV.items() if pay_tv_label == PAY_TV else CONFIG_HOMEPAGE_NO_PAYTV.items(),
            columns=[HOMEPAGE_ID, MODEL_POPULAR])
        df_popular = df_popular[df_popular[MODEL_POPULAR] == POPULAR]
        df_pay_tv = df_pay_tv[df_pay_tv[HOMEPAGE_ID].isin(df_popular[HOMEPAGE_ID])]
        df_pay_tv = df_pay_tv[~df_pay_tv[CONTENT_ID].isin(TV_CHANNEL_LIST)]
        df_sum = df_pay_tv.groupby(CONTENT_ID, as_index=False)[VIEW_COUNT].agg(SUM)
        df_sum = df_sum.sort_values(by=VIEW_COUNT, ascending=False).iloc[:20, :]
        df_sum = df_sum.merge(PopularModelFallback.get_homepage_for_contents(True if pay_tv_label == PAY_TV else False),
                              on=CONTENT_ID, how=LEFT)
        df_sum.drop(columns=VIEW_COUNT, axis=1, inplace=True)
        df_sum.drop_duplicates(CONTENT_ID, inplace=True)
        df_sum[SCORE] = NAN
        df_sum[CLUSTER_ID] = NAN
        df_sum[CREATED_ON] = datetime.utcnow().isoformat()
        all_content_based_rec = {}
        key_prefix = SERVICE_NAME + ":" + POPULAR_MODULE_DEFAULT + ":" + pay_tv_label + ":" + ALL_CONTENT_BASED + ":" + "nan"
        result = df_sum.to_json(orient=RECORDS)[1:-1]
        all_content_based_rec[key_prefix] = result
        return all_content_based_rec

    @staticmethod
    def get_default_popular_homepage_id(pay_tv_label):
        df = PopularModelFallback.get_viewed_from_s3()
        df_pay_tv = df[(df[IS_PAY_TV] == True if pay_tv_label == PAY_TV else False) & (df[CONTENT_STATUS] == ACTIVE)]
        df_pay_tv = df_pay_tv.merge(
            PopularModelFallback.get_homepage_for_contents(True if pay_tv_label == PAY_TV else False), on=CONTENT_ID,
            how=LEFT)
        df_active_homepage = DataFrame(QueryUtils.get_all_active_homepage_id_list(True, graph)[0],
                                       columns=[HOMEPAGE_ID])
        df_pay_tv = df_pay_tv.merge(df_active_homepage, on=HOMEPAGE_ID, how=LEFT)
        df_popular = DataFrame(
            CONFIG_HOMEPAGE_PAYTV.items() if pay_tv_label == PAY_TV else CONFIG_HOMEPAGE_NO_PAYTV.items(),
            columns=[HOMEPAGE_ID, MODEL_POPULAR])
        df_popular = df_popular[df_popular[MODEL_POPULAR] == POPULAR]
        df_pay_tv = df_pay_tv[df_pay_tv[HOMEPAGE_ID].isin(df_popular[HOMEPAGE_ID])]
        df_pay_tv = df_pay_tv[~df_pay_tv[CONTENT_ID].isin(TV_CHANNEL_LIST)]
        unique_homepage_id = df_pay_tv[HOMEPAGE_ID].unique()
        homepage_based_rec = {}
        for homepage_id in unique_homepage_id:
            key_prefix = SERVICE_NAME + ":" + POPULAR_MODULE_DEFAULT + ":" + pay_tv_label + ":" + HOMEPAGE_ID_BASED + ":" + "nan" + ":" + str(
                homepage_id)
            df_homepage = df_pay_tv[df_pay_tv[HOMEPAGE_ID] == homepage_id]
            df_sum = df_homepage.groupby(CONTENT_ID, as_index=False)[VIEW_COUNT].agg(SUM)
            df_sum = df_sum.sort_values(by=VIEW_COUNT, ascending=False)
            df_sum.drop(columns=VIEW_COUNT, axis=1, inplace=True)
            df_sum.drop_duplicates(CONTENT_ID, inplace=True)
            df_sum[SCORE] = NAN
            df_sum[CLUSTER_ID] = NAN
            df_sum[HOMEPAGE_ID] = homepage_id
            df_sum[CREATED_ON] = datetime.utcnow().isoformat()
            result = df_sum.to_json(orient=RECORDS)[1:-1]
            homepage_based_rec[key_prefix] = result
        return homepage_based_rec


