import time
from datetime import datetime

from pandas import DataFrame

from offline_results.common.config import VISIONPLUS_DEV, VIEWED_DATA_PATH, CONFIG_HOMEPAGE_PAYTV, \
    CONFIG_HOMEPAGE_NO_PAYTV
from offline_results.common.constants import PAY_TV_CONTENT, NO_PAY_TV_CONTENT, TV_CHANNEL_LIST, CONTENT_ID, \
    HOMEPAGE_ID, MODEL_POPULAR, POPULAR, SERVICE_NAME, ALL_CONTENT_BASED, POPULAR_MODULE_DEFAULT, RECORDS, \
    PAY_TV, LEFT, ACTIVE, CONTENT_STATUS, IS_PAY_TV, CREATED_ON, VIEW_COUNT, SUM, \
    HOMEPAGE_ID_BASED, REC_TYPE, POPULAR_FALLBACK
from offline_results.common.read_write_from_s3 import ConnectS3
from offline_results.repository.graph_db_connection import ANGraphDb
from offline_results.similarity.content_profile.query_utils import QueryUtils

graph = ANGraphDb.new_connection_config().graph


class PopularModelFallback:

    df = ConnectS3().read_compress_pickles_from_S3(bucket_name=VISIONPLUS_DEV, object_name=VIEWED_DATA_PATH,
                                                   resource=ConnectS3().create_connection())

    @staticmethod
    def get_homepage_for_contents(
            is_pay_tv
    ) -> DataFrame:
        try:
            query = f'''g.V().hasLabel('{PAY_TV_CONTENT if is_pay_tv else NO_PAY_TV_CONTENT}' ).match(
                __.as("c").values().as("content_id"),
                __.as("c").out("HAS_HOMEPAGE")
                .values("homepage_id").as("homepage_id")
                ).select("content_id", "homepage_id")'''
            graph = ANGraphDb.new_connection_config().graph
            data = graph.custom_query(query, payload={PAY_TV_CONTENT: PAY_TV_CONTENT,
                                                      NO_PAY_TV_CONTENT: NO_PAY_TV_CONTENT})
            data = [rec for idx in data for rec in idx]
            df = DataFrame(data)
            return df
        except Exception:
            graph = ANGraphDb.new_connection_config().graph
            query = f'''g.V().hasLabel('{PAY_TV_CONTENT if is_pay_tv else NO_PAY_TV_CONTENT}' ).match(
                            __.as("c").values().as("content_id"),
                            __.as("c").out("HAS_HOMEPAGE")
                            .values("homepage_id").as("homepage_id")
                            ).select("content_id", "homepage_id")'''

            data = graph.custom_query(query, payload={PAY_TV_CONTENT: PAY_TV_CONTENT,
                                                      NO_PAY_TV_CONTENT: NO_PAY_TV_CONTENT})
            data = [rec for idx in data for rec in idx]
            df = DataFrame(data)
            return df

    @staticmethod
    def get_default_popular_all_content(pay_tv_label):
        df = PopularModelFallback.df
        label = True if pay_tv_label == PAY_TV else False
        df_pay_tv = df[(df[IS_PAY_TV] == label) & (df[CONTENT_STATUS] == ACTIVE)]
        df_pay_tv = df_pay_tv.merge(PopularModelFallback.get_homepage_for_contents(label), on=CONTENT_ID, how=LEFT)
        df_popular = DataFrame(
            CONFIG_HOMEPAGE_PAYTV.items() if pay_tv_label == PAY_TV else CONFIG_HOMEPAGE_NO_PAYTV.items(),
            columns=[HOMEPAGE_ID, MODEL_POPULAR])
        df_popular = df_popular[df_popular[MODEL_POPULAR] == POPULAR]
        df_pay_tv = df_pay_tv[df_pay_tv[HOMEPAGE_ID].isin(df_popular[HOMEPAGE_ID])]
        df_pay_tv = df_pay_tv[~df_pay_tv[CONTENT_ID].isin(TV_CHANNEL_LIST)]
        df_sum = df_pay_tv.groupby(CONTENT_ID, as_index=False)[VIEW_COUNT].agg(SUM)
        df_sum = df_sum.sort_values(by=VIEW_COUNT, ascending=False).iloc[:20, :]
        df_sum = df_sum.merge(PopularModelFallback.get_homepage_for_contents(label),
                              on=CONTENT_ID, how=LEFT)
        df_sum.drop(columns=VIEW_COUNT, axis=1, inplace=True)
        df_sum.drop_duplicates(CONTENT_ID, inplace=True)
        df_sum[CREATED_ON] = datetime.utcnow().isoformat()
        df_sum[REC_TYPE] = POPULAR_FALLBACK
        all_content_based_rec = {}
        key_prefix = SERVICE_NAME + ":" + POPULAR_MODULE_DEFAULT + ":" + pay_tv_label + ":" + ALL_CONTENT_BASED
        result = df_sum.to_dict(orient=RECORDS)
        all_content_based_rec[key_prefix] = result
        return all_content_based_rec

    @staticmethod
    def get_default_popular_homepage_id(pay_tv_label):
        df = PopularModelFallback.df
        label = True if pay_tv_label == PAY_TV else False
        df_pay_tv = df[(df[IS_PAY_TV] == label) & (df[CONTENT_STATUS] == ACTIVE)]
        df_pay_tv = df_pay_tv.merge(PopularModelFallback.get_homepage_for_contents(label), on=CONTENT_ID, how=LEFT)
        df_active_homepage = DataFrame(
            QueryUtils.active_homepage_ids(PAY_TV_CONTENT if label == True else NO_PAY_TV_CONTENT),
            columns=[HOMEPAGE_ID])
        df_pay_tv = df_pay_tv.merge(df_active_homepage, on=HOMEPAGE_ID, how=LEFT)
        df_popular = DataFrame(
            CONFIG_HOMEPAGE_PAYTV.items() if pay_tv_label == PAY_TV else CONFIG_HOMEPAGE_NO_PAYTV.items(),
            columns=[HOMEPAGE_ID, MODEL_POPULAR])
        df_popular = df_popular[df_popular[MODEL_POPULAR] == POPULAR]
        df_pay_tv = df_pay_tv[df_pay_tv[HOMEPAGE_ID].isin(df_popular[HOMEPAGE_ID])]
        df_pay_tv = df_pay_tv[~df_pay_tv[CONTENT_ID].isin(TV_CHANNEL_LIST)]
        unique_homepage_id = df_pay_tv[HOMEPAGE_ID].unique()
        homepage_based_rec = {}
        for homepage_id in unique_homepage_id:
            key_prefix = SERVICE_NAME + ":" + POPULAR_MODULE_DEFAULT + ":" + pay_tv_label + ":" + HOMEPAGE_ID_BASED + ":" + str(
                homepage_id)
            df_homepage = df_pay_tv[df_pay_tv[HOMEPAGE_ID] == homepage_id]
            df_sum = df_homepage.groupby(CONTENT_ID, as_index=False)[VIEW_COUNT].agg(SUM)
            df_sum = df_sum.sort_values(by=VIEW_COUNT, ascending=False)
            df_sum.drop(columns=VIEW_COUNT, axis=1, inplace=True)
            df_sum.drop_duplicates(CONTENT_ID, inplace=True)
            df_sum[CREATED_ON] = datetime.utcnow().isoformat()
            df_sum[REC_TYPE] = POPULAR_FALLBACK
            result = df_sum.to_dict(orient=RECORDS)
            homepage_based_rec[key_prefix] = result
        return homepage_based_rec
